'============================================================================
'  Name     |  LCD Graphic Library  (Defines and Variables)                 |
'  Author   |  IKAR LLC.                                                    |
'  Version  |  1.0                                                          |
'============================================================================


'------------------------------- Font Header Byte ---------------------------------------------------
'    0    |    1    |    2    |    3    |    4    |    5    |    6    |    7    |    8    |    9    |
'----------------------------------------------------------------------------------------------------
'  First  |   Last  |            Next Font Pointer          |  Param  | Reserv  |  Width  |  Height |
'----------------------------------------------------------------------------------------------------


'------------------------------- Font Param Bits --------------------------------
'    7    |    6    |    5    |    4    |    3    |    2    |    1    |    0    |
'--------------------------------------------------------------------------------
'  Char 0 |          Type Font          |      After Char Draw Add Space X      |
'--------------------------------------------------------------------------------


'****************************************************************************
$ifndef mod_LCD_DEF
$define mod_LCD_DEF


'----------------------------------------------------------------------------
Include "Lib\Base.inc"
Include "Lib\LCD\LCD_IFC.inc"

'----------------------------------------------------------------------------
$define DMCL Word
$if _defined(mod_LCD_MONO) Or _defined(mod_LCD_MONO_DRV)
$define DMCL Byte
$endif


'----------------------------------------------------------------------------
Dim Lcd_X           As Word  Access
Dim Lcd_Y           As Word  Access
Dim Lcd_X2          As Word  Access
Dim Lcd_Y2          As Word  Access
Dim Lcd_Width       As Word  Access
Dim Lcd_Height      As Word  Access
Dim Lcd_BR          As Byte  Access
Dim Lcd_BW          As Byte  Access
Dim Lcd_BH          As Byte  Access
Dim Lcd_OfsX        As Byte  Access
Dim Lcd_OfsY        As Byte  Access
Dim Lcd_Tmp         As Byte  Access
Dim Lcd_Zoom        As Byte  Access
Dim Lcd_Cr          As DMCL  Access
Dim Lcd_Fg          As DMCL  Access
Dim Lcd_Bg          As DMCL  Access
Dim Lcd_Font        As Ptr_TBL Heap
Dim Lcd_ID          As Byte    Heap
Dim Lcd_Cfg         As Word    Heap
Dim LCD_SC          As Dword   Heap
Dim LCD_SP          As Dword   Heap
Dim Lcd_Speed       As Bit

'----------------------------------------------------------------------------
Dim GEN             As Byte System
Dim GENH            As Byte System
Dim GEN2            As Byte System
Dim GEN2H           As Byte System

'----------------------------------------------------------------------------
Dim Lcd_FontParam   As DPP1.Byte0
Dim Lcd_FontReserv  As DPP1.Byte1
Dim Lcd_FontWidth   As DPP1.Byte2
Dim Lcd_FontHeight  As DPP1.Byte3
Dim LCD_ImgPalette  As DPP1.Byte0
Dim Lcd_ImgConfig   As DPP1.Byte1
Dim Lcd_err         As DPP1.SWord1
Dim Lcd_S1          As DPP1.Word0
Dim Lcd_S2          As DPP1.Word1
Dim Lcd_S3          As DPP2.Word0
Dim Lcd_S4          As DPP2.Word1
Dim Lcd_S5          As WPP3
Dim Lcd_dx          As DPP0.Word0
Dim Lcd_dy          As DPP0.Word1


'---------------------------------- Color -----------------------------------
$define BLACK                   $0000
$define WHITE                   $FFFF
$define BLUE                    $001F
$define GREEN                   $07E0
$define RED                     $F800
$define CYAN                    $07FF
$define MAGENTA                 $F81F
$define YELLOW                  $FFE0
$define GREY                    $C618
$define DARK_GREY               $8410
$define DARK_BLUE               $0010
$define DARK_GREEN              $0400
$define DARK_RED                $8000
$define DARK_CYAN               $0410
$define DARK_MAGENTA            $8010
$define DARK_YELLOW             $8400


'----------------------------------------------------------------------------
$define LCD_T_MAIN              1
$define LCD_T_ROTN              2
$define LCD_T_OFFS              3
$define LCD_T_LIST              4
$define LCD_L_CMD               128

'----------------------------------------------------------------------------
$define LCD_CAP_INVERSE         $04
$define LCD_CAP_NOPULSE         $08
$define LCD_CAP_RESERV3         $10
$define LCD_CAP_RESERV2         $20
$define LCD_CAP_RESERV1         $40
$define LCD_CAP_RESERV0         $80

'----------------------------------------------------------------------------
' Setup initial configuration for display
' Input     : P1 holds the rotation value (0 to 3)
'           : P2 holds the address of I2C display
'           : P3 holds the capabilities of display
$define LCD_INI(P1,P2,P3)       Word (P2 << 8) + P3 + (P1 & 3)

'----------------------------------------------------------------------------
$define LCD_ROT_GET()           (Lcd_Cfg.LowByte & 3)
$define LCD_ROT_SET(P1)         WREG = P1 & 3  :  Lcd_Cfg.0 = 0  :  Lcd_Cfg.1 = 0  :  Lcd_Cfg.LowByte = Lcd_Cfg.LowByte | WREG

'----------------------------------------------------------------------------
$define LCD_OverflowX           Lcd_X  >= Lcd_Width
$define LCD_OverflowY           Lcd_Y  >= Lcd_Height
$define LCD_OverflowX2          Lcd_X2 >= Lcd_Width
$define LCD_OverflowY2          Lcd_Y2 >= Lcd_Height

'----------------------------------------------------------------------------
$define LCD_Pos(X,Y)            Lcd_X = X   :  Lcd_Y = Y
$define LCD_Print(value)        WREG = value  :  LCD_PrintChar()
$define LCD_SetFont(ptr)        Lcd_Font  = ptr
$define LCD_Corner(value)       Byte Lcd_Tmp & value <> 0

'----------------------------------------------------------------------------
$define LCD_Color               Lcd_Cr
$define LCD_ForeColor           Lcd_Fg
$define LCD_BackColor           Lcd_Bg
$define LCD_TextColor(F,B)      LCD_ForeColor = F  :  LCD_BackColor = B

'----------------------------------------------------------------------------
$define LCD_RGB_565(P1)         ((P1 >> 8) & $F800) + ((P1 >> 5) & $7E0) + ((P1 >> 3) & $1F)

'----------------------------------------------------------------------------
$define LCD_Triangle(X,Y,X2,Y2,X3,Y3)   LCD_Pos(X,Y)  :  LCD_LineTO(X2,Y2)  :  LCD_LineTO(X3,Y3)   :  LCD_LineTO(X,Y)


'----------------------------------------------------------------------------
$define TBL_Raw(P1,P2)          TBL_Next ___($defeval TBL_DTS() TBL_DTN##S##TBL_DTI)   Asm1(TBL_DTS##B)   dw (((TBL_DTS##E - TBL_DTS##B - 2)<<8) + (128*P2-0) + P1) ___
$define TBL_Next                ___($ifdef TBL_DTS)   Asm1(TBL_DTS##E)   ___($undef TBL_DTS)   ___($endif) ___($defeval TBL_DTI() $eval TBL_DTI+1) ___
$define TBL_Ptr(P1)             TBL_Raw(P1,1) CData Dword 
$define TBL_New(P1)             P1##: ___($defeval TBL_DTN() P1) ___($defeval TBL_DTI() $eval 0)
$define TBL_End                 TBL_Next CData TBL_T_END
$define TBL_T_END               255


'----------------------------------------------------------------------------
Proc TBL_Get(ptr As TBLPTRA, id As WREG)
    PB0 = id  :  DPP0.Ptr_Var = 0

    TBL_Get_Func:

    If ptr <> 0 Then
        EECON1 = 128  :  PRODH = 0

        While
            ptr = ptr + PRODH  :  PROD = cPtr16(TBLPTR)  :  If PRODL = TBL_T_END Then Break
    
            PL1 = PRODL.7  :  PRODL.7 = 0  :  If PB0 <> 255 Then If PRODL <> PB0 Then Continue
            
            PB0 = PRODL  :  If DPP0.Ptr_Var = 0 Then DPP0.Ptr_Var = ptr + PRODH
            
            If PL1 = 1 Then DPP1.Ptr_Var = cPtr32(ptr)  :  ptr = DPP1.Ptr_Var  :  GoTo TBL_Get_Func
            
            ExitProc
        Wend
    EndIf

    PRODL = TBL_T_END
EndProc


'****************************************************************************
$endif
