'============================================================================
'  Name     |  LCD Graphic Library                                          |
'  Author   |  IKAR LLC.                                                    |
'  Version  |  1.0                                                          |
'============================================================================


'****************************************************************************
$ifndef mod_LCD_GFX
$define mod_LCD_GFX


'----------------------------------------------------------------------------
$if _core <> 16
    $error "The current library for 18F devices only!"
$endif

#Disable GLCD_Low_level, Print, Cls, Cursor

Declare LCD_Type  = Graphic
Declare LCD_X_Res = 480
Declare LCD_Y_Res = 480


'----------------------------------------------------------------------------
Include "Lib\Base.inc"
Include "Lib\LCD\LCD_MONO.inc"
Include "Lib\LCD\LCD_TFT.inc"


'=================================== Init ===================================
LCD_ForeColor = BLACK  :  LCD_BackColor = BLACK  :  Lcd_Zoom = 1

'=================================== Code ===================================
GoTo Over_LCD_GFX


#ifSym __SYSCOM_GLCD_HELPERS_REQ_
__GLCD_Cursor_:
    Lcd_X.Byte1 = GENH  :  Lcd_X.Byte0 = GEN  :  Lcd_Y.Byte1 = GEN2H  :  Lcd_Y.Byte0 = GEN2
Return
#endIfSym


'----------------------------------------------------------------------------
#ifSym __SYSCOM_GLCD_PRINT_REQ_
__Print_:
Sub LCD_PrintChar()
    Lcd_Tmp = WREG   :    LCD_SC.Byte3 = WREG   :  LCD_SC.Ptr_TBL = TBLPTRA  :  TBLPTRA = Lcd_Font
    
    If Lcd_Tmp = 10 Then TABLAT = cPtr8(TBLPTRA + 9)  :  LCD_Pos(0, Lcd_Y + TABLAT)  :  ExitSub
    If Lcd_Tmp = 13 Then ExitSub
    
    Lcd_DrawChar(TBLPTRA, Lcd_Tmp)

    TBLPTRA = LCD_SC.Ptr_TBL  :  WREG = LCD_SC.Byte3
EndSub
#endIfSym


'----------------------------------------------------------------------------
Proc Lcd_DrawChar(pFont As TBLPTRA, vChar As Lcd_Tmp)
    If Lcd_Zoom = 0 Then ExitProc

    While
        If TBLPTRA = 0 Then ExitProc

        WREG = cPtr8(TBLPTR)  :  TABLAT = cPtr8(TBLPTR)  :  If Lcd_Tmp >= WREG Then Lcd_FontParam = WREG  :  If Lcd_Tmp <= TABLAT Then Break 

        $if Ptr_16
            WREG = cPtr8(TBLPTR)   :  TBLPTRH = cPtr8(TBLPTR)  :  TBLPTRL = WREG
        $else
            PROD = cPtr16(TBLPTR)  :  TBLPTRU = cPtr8(TBLPTR)  :  TBLPTR = PROD
        $endif
    Wend

    Lcd_Tmp = Lcd_FontParam                                                                             ' set real number char in font
    
    TBLPTRA = TBLPTRA + 4                                                                               ' font next ptr skip
    
    Lcd_FontParam = cPtr8(TBLPTR)  :  Lcd_FontReserv = cPtr8(TBLPTR)
    Lcd_FontWidth = cPtr8(TBLPTR)  :  Lcd_FontHeight = cPtr8(TBLPTR)

    WREG = Lcd_FontParam >> 4  :  WREG = WREG & 7  :  If WREG <> 0 Then ExitProc                        ' check type font

    If Lcd_FontParam.7 = 1 Then Lcd_Tmp = 0                                                             ' used for print unknown char

    If Lcd_FontWidth <> 0 Then                                                                          ' fixed width font
        PROD = Lcd_FontWidth * Lcd_FontHeight  :  Dec PROD  :  PROD = PROD >> 3  :  Inc PRODL           ' size bytes per char
        PROD = Lcd_Tmp * PRODL  :  TBLPTRA = TBLPTRA + PROD                                             ' offset data char  
    Else                                                                                                ' variable width font
        PROD = Lcd_Tmp * 2      :  TBLPTRA = TBLPTRA + PROD                                             ' offset table char
        PROD = cPtr16(TBLPTR)   :  TBLPTRA = TBLPTRA + PROD                                             ' offset data char 
        Lcd_FontWidth = cPtr8(TBLPTR)                                                                   ' width char 
    EndIf

    Lcd_Tmp = Lcd_ValidChar()  :  If Lcd_Tmp = 0 Then ExitProc                                          ' check overflow and init cnt for data
    
    PL0 = 0  :  If LCD_ForeColor <> LCD_BackColor Then PL0 = 1                                          ' transparent: PL0 = 0

    LCD_Glyph()  :  LCD_Color = LCD_ForeColor                                                           ' draw glyph

    WREG = Lcd_FontParam & 15  :  PROD = WREG * Lcd_Zoom                                                ' SpaceZ = SpaceX * FontZoom
    If PRODL = 0 Then ExitProc                                                                          ' if SpaceZ = 0 then exit
    If PL0 = 0 Then Lcd_X = Lcd_X + PRODL  :  ExitProc                                                  ' x = x + SpaceZ

    LCD_Color = LCD_BackColor  :  LCD_SP = DPP2                                                         ' save PP2 because its use in dec, sdec ...
    
    For Lcd_dx.Byte0 = PRODL - 1 DownTo 0
        LCD_LineV(Lcd_X, Lcd_Y, Lcd_FontHeight * Lcd_Zoom)  :  Inc Lcd_X                                ' fill SpaceZ (after end axis X char)
    Next
    
    LCD_Color = LCD_ForeColor  :  DPP2 = LCD_SP                                                         ' restore PP2
EndProc


'----------------------------------------------------------------------------
Proc Lcd_ValidChar(), Lcd_Tmp
    Result = 0
    If LCD_OverflowY Then ExitProc
    If LCD_OverflowX Then ExitProc
    If Lcd_FontWidth  = 0  Then ExitProc
    If Lcd_FontHeight = 0  Then ExitProc
    Result = 1
EndProc


'----------------------------------------------------------------------------
Proc LCD_Bitmap(pBitmap As TBLPTRA, vOpaque As PL0)
    Lcd_FontWidth  = cPtr8(pBitmap)
    Lcd_FontHeight = cPtr8(pBitmap)

    Lcd_Tmp = Lcd_ValidChar()  :  If Lcd_Tmp = 0 Then ExitProc

    LCD_SC.Word0 = LCD_Color  :  LCD_Glyph()  :  LCD_Color = LCD_SC.Word0
EndProc


'----------------------------------------------------------------------------
Sub LCD_Pixel()
    If LCD_OverflowY Then ExitSub
LCD_Pixel_X:
    If LCD_OverflowX Then ExitSub
LCD_Pixel_Fast:

$ifdef LCD_window
 Dim Lcd_wTmp As Word Access
 Dim Lcd_Wxf As Word Access
 Dim Lcd_Wyf As Word Access
 
 
 Lcd_Wxf = Lcd_Wxi + Lcd_Wwidth-1
 Lcd_Wyf = Lcd_Wyi + Lcd_WHeight-1
 
    If Lcd_X >= Lcd_Wxi And Lcd_X <= Lcd_Wxf Then
     If Lcd_Y >= Lcd_Wyi And Lcd_Y <= Lcd_Wyf Then
              Lcd_wTmp = Lcd_X - Lcd_Wxi + (Lcd_Y - Lcd_Wyi) * Lcd_Wwidth
              LCD_W[Lcd_wTmp] = LCD_Color
     End If
    End If
$endif

    LCD_Pixel_Draw
EndSub


'----------------------------------------------------------------------------
Proc LCD_LineTO(X2 As Lcd_X2, Y2 As Lcd_Y2)
    Lcd_S1 = Lcd_X2  :  Lcd_S5 = Lcd_Y2  :  LCD_Line(Lcd_X, Lcd_Y, Lcd_X2, Lcd_Y2)  :  Lcd_X = Lcd_S1   :  Lcd_Y = Lcd_S5
EndProc


'----------------------------------------------------------------------------
Proc LCD_LineV(X As Lcd_X, Y As Lcd_Y, H As Lcd_Y2)
    If LCD_OverflowX Then ExitProc

    Lcd_S3 = Lcd_Y

    If LCD_OverflowY Then
        If Lcd_Y.15 = 0 Then ExitProc
        Lcd_Y2 = Lcd_Y2 + Lcd_Y  :  If Lcd_Y2.15 = 1 Then ExitProc
        Lcd_Y  = 0
    EndIf

    If Lcd_Y2 <> 0 Then
        Lcd_S4 = Lcd_Y + Lcd_Y2  :  If Lcd_S4 > Lcd_Height Then Lcd_Y2 = Lcd_Height - Lcd_Y

        $ifdef mod_LCD_TFT
            LCD_Begin()
                LCD_setWindow(Lcd_X, Lcd_Y, 1, Lcd_Y2)  :  LCD_pushColors(Lcd_Y2, LCD_Color)
            LCD_End()
        $else
            For Lcd_Y2 = Lcd_Y2 - 1 DownTo 0
                GoSub LCD_Pixel_Fast  :  Inc Lcd_Y
            Next
        $endif
    EndIf

    Lcd_Y = Lcd_S3
EndProc


'----------------------------------------------------------------------------
Proc LCD_LineH(X As Lcd_X, Y As Lcd_Y, W As Lcd_X2)
    If LCD_OverflowY Then ExitProc

    Lcd_S3 = Lcd_X

    If LCD_OverflowX Then
        If Lcd_X.15 = 0 Then ExitProc
        Lcd_X2 = Lcd_X2 + Lcd_X  :  If Lcd_X2.15 = 1 Then ExitProc
        Lcd_X  = 0
    EndIf

    If Lcd_X2 <> 0 Then
        Lcd_S4 = Lcd_X + Lcd_X2  :  If Lcd_S4 > Lcd_Width Then Lcd_X2 = Lcd_Width - Lcd_X

        $ifdef mod_LCD_TFT
            LCD_Begin()
                LCD_setWindow(Lcd_X, Lcd_Y, Lcd_X2, 1)  :  LCD_pushColors(Lcd_X2, LCD_Color)
            LCD_End()
        $else
            For Lcd_X2 = Lcd_X2 - 1 DownTo 0
                GoSub LCD_Pixel_Fast  :  Inc Lcd_X
            Next
        $endif
    EndIf

    Lcd_X = Lcd_S3
EndProc


'----------------------------------------------------------------------------
Proc LCD_Line(X As Lcd_X, Y As Lcd_Y, X2 As Lcd_X2, Y2 As Lcd_Y2)
    If Lcd_Y = Lcd_Y2 Then                                                                                              ' draw horizontal line (fast)
        Lcd_X2 = Lcd_X2 - Lcd_X  :  If Lcd_X2 < 0 Then Lcd_X2 = -Lcd_X2  :  Lcd_X = Lcd_X - Lcd_X2  :  Dec Lcd_X
        LCD_LineH(Lcd_X, Lcd_Y, Lcd_X2 + 1)  :  ExitProc
    EndIf

    If Lcd_X = Lcd_X2 Then                                                                                              ' draw vertical   line (fast)
        Lcd_Y2 = Lcd_Y2 - Lcd_Y  :  If Lcd_Y2 < 0 Then Lcd_Y2 = -Lcd_Y2  :  Lcd_Y = Lcd_Y - Lcd_Y2  :  Dec Lcd_Y
        LCD_LineV(Lcd_X, Lcd_Y, Lcd_Y2 + 1)  :  ExitProc
    EndIf

    Lcd_dy = Lcd_Y2 - Lcd_Y  :  Lcd_dx = Lcd_X2 - Lcd_X

    Lcd_S3 = 1  :  If Lcd_dy < 0 Then Lcd_dy = -Lcd_dy  :  Set Lcd_S3
    Lcd_S4 = 1  :  If Lcd_dx < 0 Then Lcd_dx = -Lcd_dx  :  Set Lcd_S4
    
    LCD_Pixel()
                                  
    If Lcd_dx > Lcd_dy Then
        Lcd_err = Lcd_dy - Lcd_dx

        While Lcd_X <> Lcd_X2
            If Lcd_err >= 0 Then Lcd_Y = Lcd_Y + Lcd_S3  :  Lcd_err = Lcd_err - Lcd_dx
            Lcd_X = Lcd_X + Lcd_S4  :  Lcd_err = Lcd_err + Lcd_dy  :  LCD_Pixel()
        Wend
    Else
        Lcd_err = Lcd_dx - Lcd_dy

        While Lcd_Y <> Lcd_Y2
            If Lcd_err >= 0 Then Lcd_X = Lcd_X + Lcd_S4  :  Lcd_err = Lcd_err - Lcd_dy
            Lcd_Y = Lcd_Y + Lcd_S3  :  Lcd_err = Lcd_err + Lcd_dx  :  LCD_Pixel()
        Wend
    EndIf
EndProc


'----------------------------------------------------------------------------
Proc LCD_Rect(X As Lcd_X, Y As Lcd_Y, W As Lcd_dx, H As Lcd_dy)
    LCD_LineH(Lcd_X, Lcd_Y, Lcd_dx)
    LCD_LineV(Lcd_X, Lcd_Y, Lcd_dy)
    Lcd_Y = Lcd_Y + Lcd_dy  :  Dec Lcd_Y  :  LCD_LineH(Lcd_X, Lcd_Y, Lcd_dx)  :  Lcd_Y = Lcd_Y - Lcd_dy  :  Inc Lcd_Y
    Lcd_X = Lcd_X + Lcd_dx  :  Dec Lcd_X  :  LCD_LineV(Lcd_X, Lcd_Y, Lcd_dy)  :  Lcd_X = Lcd_X - Lcd_dx  :  Inc Lcd_X
EndProc


'----------------------------------------------------------------------------
Proc LCD_RectFill(X As Lcd_X, Y As Lcd_Y, W As Lcd_dx, H As Lcd_dy)
    If Lcd_dy = 0 Then ExitProc
    If Lcd_dx = 0 Then ExitProc

    If LCD_OverflowY Then If Lcd_Y.15 = 0 Then ExitProc
    If LCD_OverflowX Then If Lcd_X.15 = 0 Then ExitProc

    Lcd_S1 = Lcd_X  :  Lcd_S5 = Lcd_dx
    
    For Lcd_S5 = Lcd_S5 - 1 DownTo 0
        LCD_LineV(Lcd_X, Lcd_Y, Lcd_dy)  :  Inc Lcd_X
    Next
    
    Lcd_X = Lcd_S1
EndProc


'----------------------------------------------------------------------------
Proc LCD_Circle(X As Lcd_X, Y As Lcd_Y, R As Lcd_BR, F As WREG)
    Lcd_Tmp = WREG  :  Lcd_dx = Lcd_BR  :  If Lcd_dx = 0 Then ExitProc

    Lcd_err = 1 - Lcd_dx  :  Lcd_dy = 1  :  Lcd_S1 = Lcd_X  :  Lcd_S5 = Lcd_Y

    If LCD_Corner(15) Then 
        If LCD_Corner(6) Then Lcd_X = Lcd_S1 + Lcd_dx  :  LCD_Pixel()
        If LCD_Corner(9) Then Lcd_X = Lcd_S1 - Lcd_dx  :  LCD_Pixel()

        Lcd_X = Lcd_S1

        If LCD_Corner(12) Then Lcd_Y = Lcd_S5 + Lcd_dx  :  LCD_Pixel()
        If LCD_Corner(3)  Then Lcd_Y = Lcd_S5 - Lcd_dx  :  LCD_Pixel()
    Else
        Lcd_Tmp = Lcd_Tmp >> 4
    EndIf

    While Lcd_dx >= Lcd_dy
        Lcd_X = Lcd_S1 - Lcd_dx
        Lcd_Y = Lcd_S5 + Lcd_dy:    If Lcd_Tmp.3 = 1 Then LCD_Pixel()
        Lcd_X = Lcd_S1 + Lcd_dx:    If Lcd_Tmp.2 = 1 Then LCD_Pixel()
        Lcd_Y = Lcd_S5 - Lcd_dy:    If Lcd_Tmp.1 = 1 Then LCD_Pixel()
        Lcd_X = Lcd_S1 - Lcd_dx:    If Lcd_Tmp.0 = 1 Then LCD_Pixel()

        If Lcd_dx > Lcd_dy Then
            Lcd_X = Lcd_S1 - Lcd_dy
            Lcd_Y = Lcd_S5 + Lcd_dx:    If Lcd_Tmp.3 = 1 Then LCD_Pixel()
            Lcd_X = Lcd_S1 + Lcd_dy:    If Lcd_Tmp.2 = 1 Then LCD_Pixel()
            Lcd_Y = Lcd_S5 - Lcd_dx:    If Lcd_Tmp.1 = 1 Then LCD_Pixel()
            Lcd_X = Lcd_S1 - Lcd_dy:    If Lcd_Tmp.0 = 1 Then LCD_Pixel()
        EndIf

        PROD = Lcd_dy * 2  :  PROD = PROD + 1  :  Lcd_err = Lcd_err + PROD  :  Inc Lcd_dy

        If Lcd_err >= 0 Then Dec Lcd_dx  :  PROD = Lcd_dx * 2  :  Lcd_err = Lcd_err - PROD
    Wend

    Lcd_X = Lcd_S1  :  Lcd_Y = Lcd_S5
EndProc


'----------------------------------------------------------------------------
Proc LCD_CircleFill(X As Lcd_X, Y As Lcd_Y, R As Lcd_BR, F As WREG)
    Lcd_Tmp = WREG  :  Lcd_dx = Lcd_BR  :  If Lcd_dx = 0 Then ExitProc

    Lcd_err = 1 - Lcd_dx  :  Lcd_dy = 1  :  Lcd_S1 = Lcd_X  :  Lcd_S5 = Lcd_Y

    If LCD_Corner(15) Then
        LCD_Pixel()

        If LCD_Corner(6)  Then LCD_LineH(Lcd_S1 + 1,      Lcd_Y, Lcd_dx)
        If LCD_Corner(9)  Then LCD_LineH(Lcd_S1 - Lcd_dx, Lcd_Y, Lcd_dx)

        Lcd_X = Lcd_S1

        If LCD_Corner(12) Then LCD_LineV(Lcd_X, Lcd_S5 + 1,      Lcd_dx)
        If LCD_Corner(3)  Then LCD_LineV(Lcd_X, Lcd_S5 - Lcd_dx, Lcd_dx)
    Else
        Lcd_Tmp = Lcd_Tmp >> 4
    EndIf

    While Lcd_dx >= Lcd_dy
        Lcd_Y = Lcd_S5 + Lcd_dy
        If Lcd_Tmp.2 = 1 Then LCD_LineH(Lcd_S1 + 1,      Lcd_Y, Lcd_dx)
        If Lcd_Tmp.3 = 1 Then LCD_LineH(Lcd_S1 - Lcd_dx, Lcd_Y, Lcd_dx)

        Lcd_Y = Lcd_S5 - Lcd_dy
        If Lcd_Tmp.1 = 1 Then LCD_LineH(Lcd_S1 + 1,      Lcd_Y, Lcd_dx)
        If Lcd_Tmp.0 = 1 Then LCD_LineH(Lcd_S1 - Lcd_dx, Lcd_Y, Lcd_dx)

        PROD = Lcd_dy * 2  :  PROD = PROD + 1  :  Lcd_err = Lcd_err + PROD

        If Lcd_err >= 0 Then
            If Lcd_dx > Lcd_dy Then 
                Lcd_Y = Lcd_S5 + Lcd_dx
                If Lcd_Tmp.2 = 1 Then LCD_LineH(Lcd_S1 + 1,      Lcd_Y, Lcd_dy)
                If Lcd_Tmp.3 = 1 Then LCD_LineH(Lcd_S1 - Lcd_dy, Lcd_Y, Lcd_dy)

                Lcd_Y = Lcd_S5 - Lcd_dx
                If Lcd_Tmp.1 = 1 Then LCD_LineH(Lcd_S1 + 1,      Lcd_Y, Lcd_dy)
                If Lcd_Tmp.0 = 1 Then LCD_LineH(Lcd_S1 - Lcd_dy, Lcd_Y, Lcd_dy)
            EndIf

            Dec Lcd_dx  :  PROD = Lcd_dx * 2  :  Lcd_err = Lcd_err - PROD
        EndIf

        Inc Lcd_dy
    Wend

    Lcd_X = Lcd_S1  :  Lcd_Y = Lcd_S5
EndProc


'----------------------------------------------------------------------------
Proc LCD_Round(X As Lcd_X, Y As Lcd_Y, W As Lcd_BW, H As Lcd_BH, R As Lcd_BR)
    Lcd_S1 = Lcd_Y  :  PROD = Lcd_BR * 2  :  Lcd_dx = Lcd_BW - PROD  :  Lcd_dy = Lcd_BH - PROD

    If Lcd_dx < 0 Then Lcd_dx = 0
    If Lcd_dy < 0 Then Lcd_dy = 0

    LCD_LineH(Lcd_X+Lcd_BR,         Lcd_Y,  Lcd_dx)  :  Lcd_Y = Lcd_Y + Lcd_BH
    LCD_LineH(Lcd_X,            Lcd_Y - 1,  Lcd_dx)  :  Lcd_X = Lcd_X - Lcd_BR
    LCD_LineV(Lcd_X,        Lcd_S1+Lcd_BR,  Lcd_dy)  :  Lcd_X = Lcd_X + Lcd_BW
    LCD_LineV(Lcd_X - 1,            Lcd_Y,  Lcd_dy)  :  Lcd_X = Lcd_X - Lcd_BR

    LCD_Circle(Lcd_X,          Lcd_Y,   Lcd_BR,   32)  :  Lcd_Y = Lcd_Y + Lcd_BH  :  PROD = Lcd_BR * 2  :  Lcd_Y = Lcd_Y - PROD
    LCD_Circle(Lcd_X,      Lcd_Y - 1,   Lcd_BR,   64)  :  Lcd_X = Lcd_X - Lcd_BW  :  PROD = Lcd_BR * 2  :  Lcd_X = Lcd_X + PROD
    LCD_Circle(Lcd_X + 1,      Lcd_Y,   Lcd_BR,  128)  :  Lcd_Y = Lcd_Y - Lcd_BH  :  PROD = Lcd_BR * 2  :  Lcd_Y = Lcd_Y + PROD
    LCD_Circle(Lcd_X,      Lcd_Y + 1,   Lcd_BR,   16)  :  Lcd_X = Lcd_X - Lcd_BR  :  Lcd_Y = Lcd_Y - Lcd_BR
EndProc


'----------------------------------------------------------------------------
Proc LCD_RoundFill(X As Lcd_X, Y As Lcd_Y, W As Lcd_BW, H As Lcd_BH, R As Lcd_BR)
    PROD = Lcd_BR * 2  :  Lcd_dx = Lcd_BW - PROD

    LCD_RectFill(Lcd_X + Lcd_BR,   Lcd_Y,   Lcd_dx,  Lcd_BH)  :  Lcd_X = Lcd_X - Lcd_BR  :  PROD = Lcd_BR * 2  :  Lcd_dy = Lcd_BH - PROD
    LCD_RectFill(Lcd_X,   Lcd_Y + Lcd_BR,   Lcd_BR,  Lcd_dy)  :  Lcd_X = Lcd_X + Lcd_BW  :  Lcd_X = Lcd_X - Lcd_BR
    LCD_RectFill(Lcd_X,            Lcd_Y,   Lcd_BR,  Lcd_dy)

    LCD_CircleFill(Lcd_X - 1,      Lcd_Y,   Lcd_BR,      32)  :  Lcd_Y = Lcd_Y + Lcd_BH  :  PROD = Lcd_BR * 2  :  Lcd_Y = Lcd_Y - PROD
    LCD_CircleFill(Lcd_X,      Lcd_Y - 1,   Lcd_BR,      64)  :  Lcd_X = Lcd_X - Lcd_BW  :  PROD = Lcd_BR * 2  :  Lcd_X = Lcd_X + PROD
    LCD_CircleFill(Lcd_X + 1,      Lcd_Y,   Lcd_BR,     128)  :  Lcd_Y = Lcd_Y - Lcd_BH  :  PROD = Lcd_BR * 2  :  Lcd_Y = Lcd_Y + PROD
    LCD_CircleFill(Lcd_X,      Lcd_Y + 1,   Lcd_BR,      16)  :  Lcd_X = Lcd_X - Lcd_BR  :  Lcd_Y = Lcd_Y - Lcd_BR
EndProc


'----------------------------------------------------------------------------
Proc LCD_Gradient(value As WREG)
    Lcd_err.Byte0 = WREG  :  Lcd_err.Byte1 = WREG ^ 255                                                                 ' koef and inverse

    PRODL = LCD_ForeColor.Byte1 << 4  :  WREG  = LCD_ForeColor.Byte0 >> 4  :  WREG = PRODL | WREG  :  Rol WREG          ' calc channel - G
    WREG = WREG & $FC                 :  PROD = WREG * Lcd_err.Byte0  :  Lcd_Tmp = PRODH + 1
    PRODL = LCD_BackColor.Byte1 << 4  :  WREG  = LCD_BackColor.Byte0 >> 4  :  WREG = PRODL | WREG  :  Rol WREG
    WREG = WREG & $FC                 :  PROD = WREG * Lcd_err.Byte1  :  WREG = PRODH + Lcd_Tmp
    Ror WREG  :  Swapf WREG,W,0
    LCD_Color.Byte1 = WREG  :  LCD_Color.Byte0 = WREG & $E0  :  LCD_Color.Byte1 = LCD_Color.Byte1 & $07

    WREG = LCD_ForeColor.Byte0 & $1F  :  PROD = WREG * Lcd_err.Byte0  :  Lcd_Tmp = PRODH + 1                            ' calc channel - B
    WREG = LCD_BackColor.Byte0 & $1F  :  PROD = WREG * Lcd_err.Byte1  :  WREG = PRODH + Lcd_Tmp
    WREG = WREG & $1F  :  LCD_Color.Byte0 = LCD_Color.Byte0 | WREG

    WREG = LCD_ForeColor.Byte1 & $F8  :  PROD = WREG * Lcd_err.Byte0  :  Lcd_Tmp = PRODH + 1                            ' calc channel - R
    WREG = LCD_BackColor.Byte1 & $F8  :  PROD = WREG * Lcd_err.Byte1  :  WREG = PRODH + Lcd_Tmp
    WREG = WREG & $F8  :  LCD_Color.Byte1 = LCD_Color.Byte1 | WREG
EndProc

'----------------------------------------------------------------------------
Proc LCD_List(value As WREG)
    
    LCD_Select(value)

    While
        TBL_Get(TBLPTRA, LCD_T_LIST)    :  If PRODL <> LCD_T_LIST Then Break

        DPP1.Ptr_Var = TBLPTRA + PRODH  :  Dec DPP1.Ptr_Var
        
        While TBLPTRA < DPP1.Ptr_Var
            LCD_Begin()  :  LCD_SetCmd()
                LCD_Write(cPtr8(TBLPTR))  :  PRODL = cPtr8(TBLPTR)
                If PRODL <> 0 Then
                    $if _defined(LCD_DAT) Or _defined(LCD_PORT)
                    If PRODL.7 = 0 Then LCD_SetData()
                    $endif
                    PRODL.7 = 0  :  While PRODL <> 0  :  LCD_Write(cPtr8(TBLPTR))  :  Dec PRODL  :  Wend
                EndIf
            LCD_End()
        Wend

        TBLPTRA = DPP0
    Wend
EndProc


'----------------------------------------------------------------------------
Proc LCD_Config(value As WREG)
    
    LCD_Select(value)  :  Lcd_Cfg = 0 

    While
        TBL_Get(TBLPTRA, 255)  :  If PRODL = TBL_T_END Then Break
        
        Select Case PRODL
            Case LCD_T_MAIN
                If PRODH > 4 Then TBLPTRA = TBLPTRA + 4  :  Lcd_Cfg = cPtr16(TBLPTR)
        End Select

        TBLPTRA = DPP0
    Wend
EndProc


'----------------------------------------------------------------------------
Proc LCD_Param(value As WREG)

    LCD_Select(value)  :  Lcd_Width = 32  :  Lcd_Height = 32  :  Lcd_OfsX = 0  :  Lcd_OfsY = 0

    While
        TBL_Get(TBLPTRA, 255)  :  If PRODL = TBL_T_END Then Break
        
        Select Case PRODL
            Case LCD_T_MAIN
                Lcd_Width = cPtr16(TBLPTR)  :  Lcd_Height = cPtr16(TBLPTR)

            Case LCD_T_ROTN
                WREG = LCD_ROT_GET  :  TBLPTRA = TBLPTRA + WREG  :  Lcd_BR = cPtr8(TBLPTR)
                
            Case LCD_T_OFFS
                If PRODH > 2 Then WREG = LCD_ROT_GET  :  WREG = WREG << 1  :  TBLPTRA = TBLPTRA + WREG
                Lcd_OfsX = cPtr8(TBLPTR)  :  Lcd_OfsY = cPtr8(TBLPTR) 
        End Select

        TBLPTRA = DPP0
    Wend
    
    $ifdef LCD_User_Width
        Lcd_Width  = LCD_User_Width
    $endif

    $ifdef LCD_User_Height
        Lcd_Height = LCD_User_Height
    $endif

    $ifndef mod_LCD_MONO
        If Lcd_Cfg.0 = 1 Then Swap Lcd_Width, Lcd_Height
    $endif
EndProc


'============================================================================
Over_LCD_GFX:


'****************************************************************************
$endif
