'============================================================================
'  Name     |  LCD Graphic Library  (Interface)                             |
'  Author   |  IKAR LLC.                                                    |
'  Version  |  1.0                                                          |
'  Modified |  Added  (LCD_MISO) (LCD_MOSI)	12 nov 2022		    |
'============================================================================


'****************************************************************************
$ifndef mod_LCD_IFC
$define mod_LCD_IFC


'----------------------------------------------------------------------------
$define LCD__I2C_CMD            $00
$define LCD__I2C_DATA           $40

'----------------------------------------------------------------------------       
$define LCD_WriteWord(P1)       LCD_Write(P1.Byte1)  :  LCD_Write(P1.Byte0)

'----------------------------------------------------------------------------
$ifdef LCD_CLK_HIGH
    $define LCD_PULSE           PinClear LCD_CLK  :  PinSet   LCD_CLK
$else
    $define LCD_PULSE           PinSet   LCD_CLK  :  PinClear LCD_CLK
$endif

'----------------------------------------------------------------------------
$ifndef LCD_PORT
    $define LCD_Write(P1)       LCD_Write8(P1)
$else
    $define LCD_PULSE           PinClear LCD_WR  :  PinSet LCD_WR
    $define LCD_Write(P1)       LCD_PORT = P1  :  LCD_PULSE
    $define LCD_DirWrite        PinOutput LCD_PORT            
    $define LCD_DirRead         PinInput  LCD_PORT
$endif

'----------------------------------------------------------------------------
$ifdef LCD_MOSI 		
    $define LCD_DAT        	LCD_MOSI
$else
    $define LCD_MISO        	LCD_DAT
$endif

'----------------------------------------------------------------------------
$ifdef LCD_DAT
    $define LCD_DirWrite        PinOutput LCD_DAT            
$endif

'----------------------------------------------------------------------------
$ifdef LCD_MISO           
    $define LCD_DirRead         PinInput  LCD_MISO
$endif

'----------------------------------------------------------------------------
$ifdef LCD_CS
    $define LCD_CS_Beg          PinClear LCD_CS
    $define LCD_CS_End          PinSet   LCD_CS
$else
    $define LCD_CS_Beg
    $define LCD_CS_End
$endif

'----------------------------------------------------------------------------
$ifdef LCD_DC
    $define LCD_DC_Cmd          PinClear LCD_DC
    $define LCD_DC_Dat          PinSet   LCD_DC
$else
    $define LCD_DC_Cmd
    $define LCD_DC_Dat
$endif

'----------------------------------------------------------------------------
$ifdef LCD_DCS
    Dim ##LCD_DCS As Bit
    $define LCD_DCS_Cmd         ##LCD_DCS = 0
    $define LCD_DCS_Dat         ##LCD_DCS = 1
$else
    $define LCD_DCS_Cmd
    $define LCD_DCS_Dat
$endif

'----------------------------------------------------------------------------
$ifdef LCD_SDA
    $define LCD_I2C_Cmd         LCD_Write(LCD__I2C_CMD)
    $define LCD_I2C_Dat         LCD_Write(LCD__I2C_DATA)
$else
    $define LCD_I2C_Cmd
    $define LCD_I2C_Dat
$endif


'----------------------------------------------------------------------------
Proc LCD_IFC_Pins()
    $ifdef   LCD_PORT
        Low  LCD_PORT
    $endif
    $ifdef   LCD_RD
        High LCD_RD
    $endif
    $ifdef   LCD_WR
        High LCD_WR
    $endif

    $ifdef   LCD_CS
        High LCD_CS
    $endif
    $ifdef   LCD_DC
        Low  LCD_DC
    $endif
    $ifdef   LCD_CLK
        $ifdef LCD_CLK_HIGH
            High LCD_CLK
        $else
            Low  LCD_CLK
        $endif
    $endif
    $ifdef   LCD_DAT
        Low  LCD_DAT
    $endif
    
    $ifdef   LCD_RST
        High LCD_RST  :  DelayMS 5  :  Low LCD_RST  :  DelayMS 1  :  High LCD_RST  :  DelayMS 10
    $endif

    $ifdef    LCD_SDA
        Input LCD_SDA  :  PinClear LCD_SDA
    $endif
    $ifdef    LCD_SCL
        Input LCD_SCL  :  PinClear LCD_SCL
    $endif

    $ifdef LCD_IFC_Init
           LCD_IFC_Init
    $endif
EndProc


'----------------------------------------------------------------------------
Proc LCD_IFC_Setup(value As Lcd_Speed)
    $ifdef LCD_IFC_Speed
           LCD_IFC_Speed
    $endif
EndProc


'----------------------------------------------------------------------------
$if _defined(LCD_SDA)
    Proc LCD__Begin()
        If LCD_SDA = 0  Then  Low LCD_SCL  :  Input  LCD_SCL         ' fix if the display freezes  
        
        Input  LCD_SDA   :  Input LCD_SCL  :  Low    LCD_SDA    :    If Lcd_Speed = 0 Then LCD__I2C_Delay()
        
        DelayCS (_xtal / 10)  :  Low LCD_SCL                    :    If Lcd_Speed = 0 Then LCD__I2C_Delay()
        
        LCD_Write(Lcd_Cfg.HighByte)
    EndProc

    $define LCD_Begin()    LCD__Begin()
$else
    $define LCD_Begin()    LCD_CS_Beg
$endif


'----------------------------------------------------------------------------
$if _defined(LCD_SDA) Or _defined(LCD_IFC_Wait)
    Proc LCD__End()
        $ifdef LCD_IFC_Wait
            LCD_IFC_Wait
        $endif

        LCD_CS_End

        $ifdef LCD_SDA
            Low    LCD_SDA    :   If Lcd_Speed = 0 Then LCD__I2C_Delay()
            Input  LCD_SCL    :   If Lcd_Speed = 0 Then LCD__I2C_Delay()

            DelayCS (_xtal / 10)  :  PinInput LCD_SDA
        $endif
    EndProc

    $define LCD_End()    LCD__End()
$else
    $define LCD_End()    LCD_CS_End
$endif


'----------------------------------------------------------------------------
$if _defined(LCD_IFC_Wait)
    Proc LCD__SetCmd()
        $ifdef LCD_IFC_Wait
            LCD_IFC_Wait
        $endif

        LCD_DC_Cmd  :  LCD_DCS_Cmd  :  LCD_I2C_Cmd
    EndProc

    $define LCD_SetCmd()    LCD__SetCmd()
$else
    $define LCD_SetCmd()    LCD_DC_Cmd  :  LCD_DCS_Cmd  :  LCD_I2C_Cmd
$endif


'----------------------------------------------------------------------------
$if _defined(LCD_IFC_Wait)
    Proc LCD__SetData()
        $ifdef LCD_IFC_Wait
            LCD_IFC_Wait
        $endif

        LCD_DC_Dat  :  LCD_DCS_Dat  :  LCD_I2C_Dat
    EndProc

    $define LCD_SetData()    LCD__SetData()
$else
    $define LCD_SetData()    LCD_DC_Dat  :  LCD_DCS_Dat  :  LCD_I2C_Dat
$endif


'----------------------------------------------------------------------------
$define LCD__SPI_WBit(P1)                   '
    If P1 = 1 Then PinSet   LCD_DAT         '
    If P1 = 0 Then PinClear LCD_DAT         '
    LCD_PULSE

'----------------------------------------------------------------------------
$define LCD__I2C_WAck(value)    WREG.7 = value  :  LCD__I2C_WBit()

'----------------------------------------------------------------------------
$if _defined(LCD__I2C_OverSpeed) Or (_xtal < 48)
    $define LCD__I2C_Slow(P1)
$else
    $define LCD__I2C_Slow(P1)   DelayCS (_xtal / P1)
$endif

'----------------------------------------------------------------------------
Proc LCD__I2C_Delay()
    PCLATH = WREG  :    For WREG = (_xtal / 4) DownTo 0  :  Next    :  WREG = PCLATH
EndProc

'----------------------------------------------------------------------------
Proc LCD__I2C_WBit()
    If WREG.7 = 1 Then  Input LCD_SDA  : Else :  Low LCD_SDA

    If Lcd_Speed = 0 Then LCD__I2C_Delay()

    LCD__I2C_Slow(10)  :  Input LCD_SCL  :  LCD__I2C_Slow(6)  :  WREG.7 = LCD_SDA

    If Lcd_Speed = 0 Then LCD__I2C_Delay()

    Rol WREG  :  Low LCD_SCL
EndProc


'----------------------------------------------------------------------------
Proc LCD_Write16(value As Word)
    LCD_Write(value.Byte1)
    LCD_Write(value.Byte0)
EndProc


'----------------------------------------------------------------------------
Proc LCD_Write8(value As WREG)
    $ifdef  LCD_IFC_Wait
            LCD_IFC_Wait
    $endif

    $ifdef  LCD_IFC_Write
            LCD_IFC_Write
    $else
        $ifdef LCD_SDA
            LCD__I2C_WBit()  :  LCD__I2C_WBit()  :  LCD__I2C_WBit()  :  LCD__I2C_WBit()
            LCD__I2C_WBit()  :  LCD__I2C_WBit()  :  LCD__I2C_WBit()  :  LCD__I2C_WBit()  :  LCD__I2C_WAck(1)
        $endif
            
        $ifdef LCD_DAT
            $define LCD_SPI_SOFT

            $ifdef LCD_DCS
            LCD__SPI_WBit(LCD_DCS)
            $endif
            
            LCD__SPI_WBit(WREG.7)
            LCD__SPI_WBit(WREG.6)
            LCD__SPI_WBit(WREG.5)
            LCD__SPI_WBit(WREG.4)
            LCD__SPI_WBit(WREG.3)
            LCD__SPI_WBit(WREG.2)
            LCD__SPI_WBit(WREG.1)
            LCD__SPI_WBit(WREG.0)
        $endif
    $endif
EndProc


'----------------------------------------------------------------------------
Proc LCD_Read8(), WREG
    $ifdef  LCD_IFC_Read
            LCD_IFC_Read
    $else
        $ifdef LCD_MISO
            For PRODL = 7 DownTo 0
                Rol WREG  :  WREG.0 = 0  :  If LCD_MISO = 1 Then WREG.0 = 1
                LCD_PULSE
            Next
        $endif

        $ifdef LCD_RD
            PinClear LCD_RD  :  PinSet LCD_RD   :  WREG = LCD_PORT
        $endif
    $endif
EndProc


'****************************************************************************
$endif   
