'============================================================================
'  Name     |  LCD Monochrome display                                       |
'  Author   |  IKAR LLC.                                                    |
'  Version  |  1.0                                                          |
'============================================================================


'----------------------------------------------------------------------------
Include "Lib\LCD\LCD_MONO_DRV.inc"


'----------------------------------------------------------------------------
$if _defined(mod_LCD_MONO_DETECT) And !_defined(mod_LCD_MONO)
$define mod_LCD_MONO


'---------------------------------- Color -----------------------------------
$define BLACK                   $00
$define WHITE                   $FF
$define BLUE                    WHITE
$define GREEN                   WHITE
$define RED                     WHITE
$define CYAN                    WHITE
$define MAGENTA                 WHITE
$define YELLOW                  WHITE
$define GREY                    WHITE
$define DARK_GREY               WHITE
$define DARK_BLUE               WHITE
$define DARK_GREEN              WHITE
$define DARK_RED                WHITE
$define DARK_CYAN               WHITE
$define DARK_MAGENTA            WHITE
$define DARK_YELLOW             WHITE


'------------------------------ Use in GFX Lib ------------------------------
$define LCD_Pixel_Pos                                                                               '
    Lfsr 2, (Lcd_Mask)  :  WREG = Lcd_Y.Byte0 & 7  :  Lcd_Mask_Y = PLUSW2  :  Lfsr 2, (Lcd_Buf)     '
    Asm ___ Rlncf Lcd_Y,W,0 ___ Swapf WREG,W,0 ___ Andlw 15 ___ EndAsm                              '
    PROD = WREG * Lcd_Width.LowByte  :  FSR2 = FSR2 + PROD  :  FSR2 = FSR2 + Lcd_X.Byte0

'------------------------------ Use in GFX Lib ------------------------------
$define LCD_Pixel_Color         WREG = LCD_Color.LowByte  :  WREG = INDF2 ^ WREG   :  WREG = Lcd_Mask_Y & WREG  :  POSTINC2 = POSTINC2 ^ WREG
$define LCD_Pixel_Row(P1)       FSR2 = FSR2 - P1 ___ Asm ___ Rlncf Lcd_Mask_Y,F,0 ___ EndAsm ___ If Lcd_Mask_Y.0 = 1 Then FSR2 = FSR2 + Lcd_Width.LowByte
$define LCD_Pixel_Draw          LCD_Pixel_Pos  :  LCD_Pixel_Color


'----------------------------------------------------------------------------
Dim Lcd_Mask_Y          As Byte Access
Dim Lcd_Mask[8]         As Byte Heap
Dim Lcd_Buf[LCD_SIZE]   As Byte Heap


'=================================== Init ===================================
Lfsr 2, (Lcd_Mask)  :  WREG = 1  :  Repeat  :  POSTINC2 = WREG  :  WREG = WREG << 1  :  Until WREG = 0            ' Set mask array


'----------------------------------------------------------------------------
Proc LCD_Init()
    LCD_IFC_Pins()
    LCD_IFC_Setup(0)                        ' baudrate for init
    
    LCD_Config(0)
    LCD_List(0)
    
    LCD_Invert(False)    
    LCD_Rotation(LCD_ROT_GET)

    LCD_IFC_Setup(1)                        ' baudrate for work
EndProc


'----------------------------------------------------------------------------
Proc LCD_Contrast(value As WREG)
    PRODL = value

    LCD_Begin()  :  LCD_SetCmd()

    $ifdef LCD_USE_SSD1306
        If Lcd_ID < LCD_ST7565 Then
            LCD_Write(SSD1306_CONTRAST)  :  LCD_Write(PRODL)
        EndIf
    $endif

    $ifdef LCD_USE_HX1230
        If Lcd_ID = LCD_HX1230 Then
            PRODL = PRODL & $1F  :  LCD_Write($20)  :  LCD_Write($80 | PRODL)
        EndIf
    $endif

    $ifdef LCD_USE_ST7565
        If Lcd_ID = LCD_ST7565 Then
            PRODL = PRODL & $3F  :  LCD_Write(SSD1306_CONTRAST)  :  LCD_Write(ST7565_CONTRAST_ARG | PRODL)
        EndIf
    $endif

    $ifdef LCD_USE_NOK5110
        If Lcd_ID = LCD_NOK5110 Then
            PRODL = PRODL & $7F  :  LCD_Write(NOK5110_FUNC_EXT)  :  LCD_Write(NOK5110_VOP | PRODL)  :  LCD_Write(NOK5110_FUNC)
        EndIf
    $endif

    LCD_End()
EndProc


'----------------------------------------------------------------------------
Proc LCD_Invert(value As WREG)
    PRODL = value

    LCD_Begin()  :  LCD_SetCmd()

    $if _defined(LCD_USE_SSD1306) Or _defined(LCD_USE_HX1230) Or _defined(LCD_USE_ST7565)
        If Lcd_ID <= LCD_HX1230 Then
            If PRODL = False Then
                LCD_Write(SSD1306_INVERT_OFF)
            Else
                LCD_Write(SSD1306_INVERT_ON)
            EndIf
        EndIf
    $endif

    $ifdef LCD_USE_NOK5110
        If Lcd_ID = LCD_NOK5110 Then
            If PRODL = False Then
                LCD_Write(NOK5110_DISPLAY | NOK5110_NORMAL)
            Else
                LCD_Write(NOK5110_DISPLAY | NOK5110_INVERT)
            EndIf
        EndIf
    $endif

    LCD_End()
EndProc


'----------------------------------------------------------------------------
Proc LCD_Rotation(value As WREG)
    LCD_ROT_SET(value)  :  LCD_Param(0)

    LCD_Begin()  :  LCD_SetCmd()

    $ifdef LCD_USE_SSD1306
        If Lcd_ID < LCD_ST7565 Then
            If Lcd_Cfg.1 = 0 Then
                LCD_Write(SSD1306_REMAP_L_R)  :  LCD_Write(SSD1306_REMAP_T_D)
            Else
                LCD_Write(SSD1306_REMAP_R_L)  :  LCD_Write(SSD1306_REMAP_D_T)
            EndIf
        EndIf
    $endif

    $ifdef LCD_USE_HX1230
        If Lcd_ID = LCD_HX1230 Then
            If Lcd_Cfg.1 = 0 Then
                LCD_Write(SSD1306_REMAP_R_L)  :  LCD_Write(SSD1306_REMAP_D_T)
            Else
                LCD_Write(SSD1306_REMAP_L_R)  :  LCD_Write(SSD1306_REMAP_T_D)
            EndIf
        EndIf
    $endif

    $ifdef LCD_USE_ST7565
        If Lcd_ID = LCD_ST7565 Then
            If Lcd_Cfg.1 = 0 Then
                LCD_Write(SSD1306_REMAP_R_L)  :  LCD_Write(SSD1306_REMAP_T_D)
            Else
                LCD_Write(SSD1306_REMAP_L_R)  :  LCD_Write(SSD1306_REMAP_D_T)
            EndIf
        EndIf
    $endif

    LCD_End()
EndProc


'----------------------------------------------------------------------------
Proc LCD_Clear()
    LCD_Fill(BLACK)
EndProc


'----------------------------------------------------------------------------
Proc LCD_Fill(value As LCD_Color)
    WREG = Lcd_Height.LowByte + 7  :  WREG = WREG & (~7)  :  PROD = WREG * Lcd_Width.LowByte  :  PROD = PROD << 2
    
    WREG = LCD_Color.LowByte  :  Lfsr 2, (Lcd_Buf)
    
    For PRODH = PRODH - 1 DownTo 0
        POSTINC2 = WREG  :  POSTINC2 = WREG  :  POSTINC2 = WREG  :  POSTINC2 = WREG
        POSTINC2 = WREG  :  POSTINC2 = WREG  :  POSTINC2 = WREG  :  POSTINC2 = WREG
    Next

    LCD_Color = LCD_ForeColor
EndProc


'----------------------------------------------------------------------------
Proc LCD_Update()
    $if _defined(LCD_USE_SSD1306) Or _defined(LCD_USE_HX1230) Or _defined(LCD_USE_ST7565)
        If Lcd_ID <= LCD_HX1230 Then
            Lcd_Tmp = 0  :  PRODH = Lcd_Height.LowByte + 7  :  PRODH = PRODH / 8  :  Lfsr 2, (Lcd_Buf)
            
            For PRODH = PRODH - 1 DownTo 0
                LCD_Begin()  :  LCD_SetCmd()
                    WREG = Lcd_OfsY + Lcd_Tmp  :  LCD_Write(WREG | SSD1306_ADDR_ONE)            ' Page address
                    WREG = Lcd_OfsX & $0F      :  LCD_Write(WREG | SSD1306_ADDR_COL_L)          ' Lower  column address
                    WREG = Lcd_OfsX / 16       :  LCD_Write(WREG | SSD1306_ADDR_COL_H)          ' Higher column address
                LCD_End()

                LCD_Begin()  :  LCD_SetData()
                    For PRODL = Lcd_Width.LowByte - 1 DownTo 0
                        LCD_Write(POSTINC2)
                    Next
                LCD_End()

                Inc Lcd_Tmp
            Next
        EndIf
    $endif

    $ifdef LCD_USE_NOK5110
        If Lcd_ID = LCD_NOK5110 Then
            Lfsr 2, (Lcd_Buf)

            LCD_Begin()  :  LCD_SetCmd()
                LCD_Write(NOK5110_ADDR_Y)  :  LCD_Write(NOK5110_ADDR_X)
            LCD_End()

            LCD_Begin()  :  LCD_SetData()
                For PRODL = (((84 * 48) / 8) / 4) - 1 DownTo 0
                    LCD_Write(POSTINC2)  :  LCD_Write(POSTINC2)  :  LCD_Write(POSTINC2)  :  LCD_Write(POSTINC2)
                Next
            LCD_End()
            
            LCD_Begin()  :  LCD_SetCmd()
                LCD_Write(NOK5110_ADDR_Y)  :  LCD_Write(NOK5110_ADDR_X)
            LCD_End()
        EndIf
    $endif
EndProc


'----------------------------------------------------------------------------
Proc LCD_Glyph_Rect()
    LCD_Pixel_Pos
    
    For Lcd_dy.Byte1 = Lcd_Zoom - 1 DownTo 0
        
        For Lcd_dx.Byte1 = Lcd_Zoom - 1 DownTo 0
            LCD_Pixel_Color
        Next

        LCD_Pixel_Row(Lcd_Zoom)
    Next
EndProc


'----------------------------------------------------------------------------
Proc LCD_Glyph()
    PROD = Lcd_FontWidth  * Lcd_Zoom  :  Lcd_X2 = Lcd_X + PROD  :  If LCD_OverflowX2 Then ExitProc
    PROD = Lcd_FontHeight * Lcd_Zoom  :  Lcd_Y2 = Lcd_Y + PROD  :  If LCD_OverflowY2 Then ExitProc
        
    '------------------------------ Draw Zoom -------------------------------
    If Lcd_Zoom > 1 Then

        For Lcd_dy.Byte0 = Lcd_FontHeight - 1 DownTo 0
            For Lcd_dx.Byte0 = Lcd_FontWidth - 1 DownTo 0
            
                Dec Lcd_Tmp  :  If Lcd_Tmp = 0 Then Tblrd*+  :  Lcd_Tmp.3 = 1

                If TABLAT.0 = 1 Then
                    LCD_Color = LCD_ForeColor  :  LCD_Glyph_Rect()
                ElseIf PL0 = 1 Then
                    LCD_Color = LCD_BackColor  :  LCD_Glyph_Rect()
                EndIf
                
                Ror TABLAT  :  Lcd_X = Lcd_X + Lcd_Zoom
            Next

            Lcd_Y = Lcd_Y + Lcd_Zoom  :  PROD = Lcd_FontWidth * Lcd_Zoom  :  Lcd_X = Lcd_X - PROD
        Next

        PROD = Lcd_FontHeight * Lcd_Zoom  :  Lcd_Y = Lcd_Y - PROD

    '---------------------------- Draw Simple -----------------------------
    Else

        LCD_Pixel_Pos
        
        For Lcd_dy.Byte0 = Lcd_FontHeight - 1 DownTo 0
            For Lcd_dx.Byte0 = Lcd_FontWidth - 1 DownTo 0
                                                        
                Dec Lcd_Tmp  :  If Lcd_Tmp = 0 Then Tblrd*+  :  Lcd_Tmp.3 = 1                    ' read next data and counter = 8
    
                If TABLAT.0 = 1 Then                                                            ' if pixel = on
                    LCD_Color = LCD_ForeColor  :  LCD_Pixel_Color
                ElseIf PL0 = 1 Then                                                             ' if pixel = off and forecolor <> backcolor
                    LCD_Color = LCD_BackColor  :  LCD_Pixel_Color
                Else
                    WREG = POSTINC2
                EndIf
                
                Ror TABLAT                                                                      ' shift data to right by 1
            Next
            
            LCD_Pixel_Row(Lcd_FontWidth)
        Next
        
    EndIf
    
    '---------------------------- Set end X -----------------------------
    PROD = Lcd_FontWidth * Lcd_Zoom  :  Lcd_X = Lcd_X + PROD
EndProc



'****************************************************************************
$endif
