'============================================================================
'  Name     |  LCD Color display                                            |
'  Author   |  IKAR LLC.                                                    |
'  Version  |  1.0                                                          |
'============================================================================


'----------------------------------------------------------------------------
Include "Lib\LCD\LCD_DEF.inc"
Include "Lib\LCD\LCD_TFT_DRV.inc"


'----------------------------------------------------------------------------
$if _defined(mod_LCD_TFT_DETECT) And !_defined(mod_LCD_TFT)
$define mod_LCD_TFT

'----------------------------------------------------------------------------
$ifdef mod_LCD_MONO
$error "You cannot simultaneously use a monochrome and color display."
$endif


'--------------------------------- Specific ---------------------------------
$define LCD_setAddrY(P1)        LCD_SetCmd()  :  LCD_Write(LCD_CMD_RASET)  :  LCD_SetData()  :  PROD = Lcd_Y + Lcd_OfsY  :  LCD_WriteWord(PROD)  :  P1  :  LCD_WriteWord(PROD)
$define LCD_setAddrX(P1)        LCD_SetCmd()  :  LCD_Write(LCD_CMD_CASET)  :  LCD_SetData()  :  PROD = Lcd_X + Lcd_OfsX  :  LCD_WriteWord(PROD)  :  P1  :  LCD_WriteWord(PROD)
$define LCD_setAddrM(P1)        LCD_SetCmd()  :  LCD_Write(LCD_CMD_CASET)  :  LCD_SetData()  :  PROD = Lcd_X + Lcd_OfsX  :  LCD_WriteWord(PROD)  :  LCD_WriteWord(P1)  


'------------------------------ Use in GFX Lib ------------------------------
$define LCD_Pixel_Draw                                                      '
LCD_Begin()                                                                 '
    LCD_setAddrY                                                            '
    LCD_setAddrX                                                            '
    LCD_SetCmd()   :  LCD_Write(LCD_CMD_RAMWR)                              '
    LCD_SetData()  :  LCD_WriteWord(LCD_Color)                              '
LCD_End()


'----------------------------------------------------------------------------
Proc LCD_Init()
    LCD_IFC_Pins()
    LCD_IFC_Setup(0)                                                                                                    ' baudrate for init

    LCD_Config(0)
    
    $ifndef LCD_RST
    LCD_Begin()  :  LCD_SetCmd()  :  LCD_Write(LCD_CMD_SWRESET)  :  LCD_End()      :  DelayMS 100                       ' Software reset
    $endif
    
    LCD_Begin()  :  LCD_SetCmd()  :  LCD_Write(LCD_CMD_SLPOUT)   :  LCD_End()      :  DelayMS 120                       ' Exit sleep
    LCD_Begin()  :  LCD_SetCmd()  :  LCD_Write(LCD_CMD_DISPOFF)  :  LCD_End()                                           ' Display off
    LCD_Begin()  :  LCD_SetCmd()  :  LCD_Write(LCD_CMD_PIXFMT)   :  LCD_SetData()  :  LCD_Write($55)  :  LCD_End()      ' Pixel read=565, write=565.

    LCD_List(0)
    
    LCD_Begin()  :  LCD_SetCmd()  :  LCD_Write(LCD_CMD_DISPON)   :  LCD_End()       :  DelayMS 10                       ' Display On
 
    LCD_Invert(False)
    LCD_Rotation(LCD_ROT_GET)

    LCD_IFC_Setup(1)                                                                                                    ' baudrate for work
EndProc


'----------------------------------------------------------------------------
Proc LCD_Invert(value As WREG)
    If value = False Then PRODL = LCD_CMD_DINVOF  : Else :  PRODL = LCD_CMD_DINVON
    
    WREG = Lcd_Cfg.LowByte & LCD_CAP_INVERSE
    
    If WREG <> 0 Then PRODL.0 =~ PRODL.0
    
    LCD_Begin()
        LCD_SetCmd()  :  LCD_Write(PRODL)
    LCD_End()
EndProc


'----------------------------------------------------------------------------
Proc LCD_Rotation(value As WREG)
    LCD_ROT_SET(value)  :  LCD_Param(0)
    
    LCD_Begin()
        LCD_SetCmd()   :  LCD_Write(LCD_CMD_MADCTL)
        LCD_SetData()  :  LCD_Write(Lcd_BR)
    LCD_End()
EndProc


'----------------------------------------------------------------------------
Proc LCD_Contrast(value As WREG)
EndProc

'----------------------------------------------------------------------------
Proc LCD_Update()
EndProc


'----------------------------------------------------------------------------
Proc LCD_Clear()
    LCD_Fill(BLACK)
EndProc


'----------------------------------------------------------------------------
Proc LCD_Fill(value As LCD_Color)
    LCD_Begin()
        LCD_setWindow(0, 0, Lcd_Width, Lcd_Height)
        
        For Lcd_Y2 = Lcd_Y2 - 1 DownTo 0
            LCD_pushColors(Lcd_X2, LCD_Color)
        Next
    LCD_End()

    LCD_Color = LCD_ForeColor
EndProc


'----------------------------------------------------------------------------
Proc LCD_fastPixels()
    For PRODH = PRODH DownTo 0
        For PRODL = PRODL DownTo 0
            LCD_PULSE  :  LCD_PULSE
            $ifdef LCD_SPI_SOFT
            LCD_PULSE  :  LCD_PULSE  :  LCD_PULSE  :  LCD_PULSE  :  LCD_PULSE  :  LCD_PULSE  :  LCD_PULSE
            LCD_PULSE  :  LCD_PULSE  :  LCD_PULSE  :  LCD_PULSE  :  LCD_PULSE  :  LCD_PULSE  :  LCD_PULSE
            $endif
        Next
    Next
EndProc


'----------------------------------------------------------------------------
Proc LCD_pushColors(cnt As PROD, value As LCD_Color)
    Dec PROD

    $if _defined(LCD_PORT) Or _defined(LCD_SPI_SOFT)
        If LCD_Color.Byte0 = LCD_Color.Byte1 Then
            $ifdef LCD_PORT
                LCD_PORT = LCD_Color.Byte0  :  LCD_fastPixels()  :  ExitProc
            $else   
                If LCD_Color.Byte0 = 0   Then PinClear LCD_DAT  :  LCD_fastPixels()  :  ExitProc
                If LCD_Color.Byte0 = 255 Then PinSet   LCD_DAT  :  LCD_fastPixels()  :  ExitProc
            $endif
        EndIf
    $endif

    For PRODH = PRODH DownTo 0
        For PRODL = PRODL DownTo 0
            LCD_WriteWord(LCD_Color) 
        Next
    Next
EndProc


'----------------------------------------------------------------------------
Proc LCD_setWindow(X As Lcd_X, Y As Lcd_Y, W As Lcd_X2, H As Lcd_Y2)
    LCD_setAddrY( PROD = PROD + Lcd_Y2  :  Dec PROD )
    LCD_setAddrX( PROD = PROD + Lcd_X2  :  Dec PROD )
    LCD_SetCmd()  :  LCD_Write(LCD_CMD_RAMWR)  :  LCD_SetData()
EndProc


'----------------------------------------------------------------------------
Proc LCD_Glyph()
    LCD_Begin()

    '------------------------------ Draw Zoom -------------------------------
    If Lcd_Zoom > 1 Then

        Lcd_X2 = Lcd_Zoom  :  Lcd_Y2 = Lcd_Zoom

        For Lcd_dy.Byte0 = Lcd_FontHeight - 1 DownTo 0
            For Lcd_dx.Byte0 = Lcd_FontWidth - 1 DownTo 0

                Dec Lcd_Tmp  :  If Lcd_Tmp = 0 Then TABLAT = cPtr8(TBLPTR)  :  Lcd_Tmp.3 = 1

                If TABLAT.0 = 1 Then
                    LCD_setWindow(Lcd_X, Lcd_Y, Lcd_X2, Lcd_Y2)  :  LCD_pushColors(Lcd_Zoom * Lcd_Zoom, LCD_ForeColor)
                ElseIf PL0 = 1 Then
                    LCD_setWindow(Lcd_X, Lcd_Y, Lcd_X2, Lcd_Y2)  :  LCD_pushColors(Lcd_Zoom * Lcd_Zoom, LCD_BackColor)
                EndIf

                Ror TABLAT  :  Lcd_X = Lcd_X + Lcd_Zoom
            Next

            Lcd_Y = Lcd_Y + Lcd_Zoom  :  PROD = Lcd_FontWidth * Lcd_Zoom  :  Lcd_X = Lcd_X - PROD
        Next

        PROD = Lcd_FontHeight * Lcd_Zoom  :  Lcd_Y = Lcd_Y - PROD

    '----------------------------- Draw Opaque ------------------------------
    ElseIf PL0 = 1 Then

        LCD_setWindow(Lcd_X, Lcd_Y, Lcd_FontWidth, Lcd_FontHeight)  :  PROD = Lcd_FontWidth * Lcd_FontHeight

        For PROD = PROD - 1 DownTo 0
            Dec Lcd_Tmp  :  If Lcd_Tmp = 0 Then TABLAT = cPtr8(TBLPTR)  :  Lcd_Tmp.3 = 1

            If TABLAT.0 = 1 Then  LCD_WriteWord(LCD_ForeColor)  : Else :  LCD_WriteWord(LCD_BackColor)

            Ror TABLAT
        Next

    '--------------------------- Draw Transparent ---------------------------
    Else

        Lcd_dx = Lcd_OfsX + Lcd_Width  :  Dec Lcd_dx
        
        For Lcd_dy.Byte0 = Lcd_FontHeight - 1 DownTo 0

            LCD_setAddrY  :  PL1 = 0

            For Lcd_dy.Byte1 = Lcd_FontWidth - 1 DownTo 0

                Dec Lcd_Tmp  :  If Lcd_Tmp = 0 Then TABLAT = cPtr8(TBLPTR)  :  Lcd_Tmp.3 = 1

                If TABLAT.0 = 1 Then 
                    If PL1 = 0 Then LCD_setAddrM(Lcd_dx)  :  LCD_SetCmd()  :  LCD_Write(LCD_CMD_RAMWR)  :  LCD_SetData()
                    LCD_WriteWord(LCD_ForeColor)  :  PL1 = 1
                Else
                    PL1 = 0
                EndIf

                Ror TABLAT  :  Inc Lcd_X
            Next

            Inc Lcd_Y  :  Lcd_X = Lcd_X - Lcd_FontWidth
        Next

        Lcd_Y = Lcd_Y - Lcd_FontHeight
    EndIf

    PROD = Lcd_FontWidth * Lcd_Zoom  :  Lcd_X = Lcd_X + PROD

    LCD_End()
EndProc


'----------------------------------------------------------------------------
Proc LCD_Image(ptr As TBLPTRA)
    Lcd_X2 = cPtr16(TBLPTR)                                                     ' width
    Lcd_Y2 = cPtr16(TBLPTR)                                                     ' height
    Lcd_S1 = cPtr16(TBLPTR)                                                     ' config and palette

    If LCD_OverflowY Then ExitProc                                              ' overflow Y
    If LCD_OverflowX Then ExitProc                                              ' overflow X
    If Lcd_X2 = 0    Then ExitProc
    If Lcd_Y2 = 0    Then ExitProc

    LCD_Begin()
    
    LCD_setWindow(Lcd_X, Lcd_Y, Lcd_X2, Lcd_Y2)

    '----------------------------- Draw 16 bit ------------------------------
    If Lcd_ImgConfig.2 = 1 Then

        For Lcd_Y2 = Lcd_Y2 - 1 DownTo 0
            For Lcd_S5 = Lcd_X2 - 1 DownTo 0
                LCD_Color = cPtr16(TBLPTR)  :  LCD_WriteWord(LCD_Color)
            Next
        Next

    '----------------------------- Draw 8 bit ------------------------------
    ElseIf Lcd_ImgConfig.1 = 1 Then
        
        LCD_Palette_Init()      
        
        Do
            TABLAT = cPtr8(TBLPTR)  :  PROD = 1

            If TABLAT >= $C0 Then
                PRODL = TABLAT & (~$C0)
                
                If PRODL = 0 Then
                    TABLAT = cPtr8(TBLPTR)  :  If TABLAT = 0 Then Break
                    If TABLAT < 64 Then PRODH = TABLAT  :  TABLAT = cPtr8(TBLPTR)
                    PRODL = TABLAT
                EndIf

                TABLAT = cPtr8(TBLPTR)
            EndIf

            LCD_pushColors(PROD, LCD_Palette_Get())
        Loop

    '----------------------------- Draw 4 bit ------------------------------
    ElseIf Lcd_ImgConfig.0 = 1 Then

        LCD_Palette_Init()

        Do
            TABLAT = LCD_Image_HByte()  :  PROD = 1

            If TABLAT >= $C Then
                PRODL = TABLAT & (~$C)

                If PRODL = 0 Then
                    TABLAT = LCD_Image_FByte()  :  If TABLAT = 0 Then Break
                    If TABLAT < 4 Then PRODH = TABLAT  :  TABLAT = LCD_Image_FByte()
                    PRODL = TABLAT
                EndIf

                TABLAT = LCD_Image_HByte()
            EndIf

            LCD_pushColors(PROD, LCD_Palette_Get())
        Loop
        
    EndIf

    LCD_End()

    LCD_Color = LCD_ForeColor
EndProc


'----------------------------------------------------------------------------
Proc LCD_Palette_Init()
    LCD_SC.Ptr_TBL = TBLPTRA  :  PROD = (LCD_ImgPalette + 1)  :  PROD = PROD * 2  :  TBLPTRA = TBLPTRA + PROD  :  PL1 = 0
EndProc


'----------------------------------------------------------------------------
Proc LCD_Palette_Get(), LCD_Color
    LCD_SP.Ptr_TBL = TBLPTRA  :  TBLPTRA = TABLAT << 1  :  Result = cPtr16(TBLPTRA + LCD_SC.Ptr_TBL)  :  TBLPTRA = LCD_SP.Ptr_TBL
EndProc


'----------------------------------------------------------------------------
Proc LCD_Image_HByte(), TABLAT
    If PL1 = 1 Then TABLAT = Lcd_Tmp >> 4  : Else :  WREG = cPtr8(TBLPTR)  :  Lcd_Tmp = WREG  :  TABLAT = WREG & 15
    PL1 =~ PL1
EndProc


'----------------------------------------------------------------------------
Proc LCD_Image_FByte(), TABLAT
    If PL1 = 0 Then TABLAT = cPtr8(TBLPTR)  :  ExitProc
    Lcd_Tmp = Lcd_Tmp >> 4  :  WREG = cPtr8(TBLPTR)  :  TABLAT = Lcd_Tmp  :  Lcd_Tmp = WREG  :  WREG = WREG << 4  :  TABLAT = TABLAT | WREG
EndProc


'----------------------------------------------------------------------------
Proc LCD_ReadReg(reg As PRODH, cnt As Lcd_Tmp, dummy As PRODL), Dword
    LCD_Begin()
        LCD_SetCmd()  :  LCD_Write(reg)  :  LCD_DirRead   :  LCD_SetData()

        Result = 0  :  While dummy <> 0  :  LCD_PULSE  :  Dec dummy  :  Wend

        For cnt = cnt - 1 DownTo 0
            Result.Byte3 = Result.Byte2  :  Result.Byte2 = Result.Byte1  :  Result.Byte1 = Result.Byte0  :  Result.Byte0 = LCD_Read8()
        Next
    LCD_End()

    LCD_DirWrite
EndProc


'----------------------------------------------------------------------------
Proc LCD_WriteRAM(ByRef ptrData As FSR2, pX As Lcd_X, pY As Lcd_Y, pW As Lcd_X2, pH As Lcd_Y2)
    LCD_Begin()
        LCD_setWindow(pX, pY, pW, pH)  :  Lcd_S5 = pW.Byte0 * pH.Byte0

        For Lcd_S5 = Lcd_S5 - 1 DownTo 0
            PRODL = POSTINC2  :  PRODH = POSTINC2  :  LCD_WriteWord(PROD)
        Next   
    LCD_End()
EndProc


'----------------------------------------------------------------------------
Proc LCD_ReadRAM(ByRef ptrData As FSR2, pX As Lcd_X, pY Lcd_Y, pW As Lcd_X2, pH As Lcd_Y2)

$ifndef  ST7735

    LCD_Begin()
        LCD_SetCmd()  :  LCD_Write(LCD_CMD_PIXFMT)  :  LCD_SetData()  :  LCD_Write($66)
    LCD_End()
      
    LCD_Begin()
        LCD_setWindow(pX, pY, pW, pH)  :  Lcd_S5 = pW.Byte0 * pH.Byte0

        LCD_SetCmd() :  LCD_Write(LCD_CMD_RAMRD) :  LCD_DirRead  :  LCD_SetData()  :  LCD_Read8()
        
        WREG = Lcd_Cfg.LowByte & LCD_CAP_NOPULSE  :  If WREG = 0 Then LCD_PULSE
       
        For Lcd_S5 = Lcd_S5 - 1 DownTo 0

            WREG = LCD_Read8()  :  WREG = WREG & $F8  :  Lcd_S1.Byte1 = WREG
            WREG = LCD_Read8()  :  WREG = WREG & $FC  :  Lcd_S2 = WREG * 8
            WREG = LCD_Read8()  :  Lcd_S1.Byte0 = WREG >> 3
            
            POSTINC2 = Lcd_S1.Byte0 | Lcd_S2.Byte0
            POSTINC2 = Lcd_S1.Byte1 | Lcd_S2.Byte1
        Next

    LCD_End()
    
    LCD_DirWrite     

    LCD_Begin()
        LCD_SetCmd()  :  LCD_Write(LCD_CMD_PIXFMT)  :  LCD_SetData()  :  LCD_Write($55)
    LCD_End()
    
          $else

    LCD_Begin()
        LCD_SetCmd()  :  LCD_Write(LCD_CMD_PIXFMT)  :  LCD_SetData()  :  LCD_Write($66)
    LCD_End() 

    If LCD_rot > 1 Then    
                        Select Case LCD_rot
                        
                            Case 2
                        
                             LCD_Rotation(0)
                             LCD_Begin()
                        
                             If Lcd_Width = 128 Then
                                                     LCD_setWindow(Lcd_Width + 4 - pW - PX, Lcd_Height + 2 - pH - PY, pW, pH) 
                                                Else
                                                     LCD_setWindow(Lcd_Width - 2 - pW - PX, Lcd_Height - pH - PY, pW, pH) : Lcd_S5 = pW.Byte0 * pH.Byte0
                             End If                   
                        
                            Case 3
                            
                             LCD_Rotation(1) 
                             LCD_Begin()
                            
                             If Lcd_Height = 128 Then  
                                                      LCD_setWindow(Lcd_Width + 2 - pW - PX,Lcd_Height + 4 - pH - PY, pW, pH)
                                                 Else
                                                      LCD_setWindow(Lcd_Width - pW - PX,Lcd_Height - 2 - pH - PY, pW, pH)                 
                            End If
                                          
                        End Select
                  Else
                        LCD_Begin()
                        LCD_setWindow(pX, pY, pW, pH)            
    End If 
    
    Lcd_S5 = pW.Byte0 * pH.Byte0                    
         
    LCD_SetCmd() :  LCD_Write(LCD_CMD_RAMRD) :  LCD_DirRead  :  LCD_SetData()  :  LCD_Read8()
        
    LCD_Read8()  :  LCD_PULSE
        
    WREG = Lcd_Cfg.LowByte & LCD_CAP_NOPULSE  :  If WREG = 0 Then LCD_PULSE
           
    If LCD_rot > 1 Then
     
                        FSR2 = FSR2 + (Lcd_S5 * 2) - 1
      
                        For Lcd_S5 = Lcd_S5 - 1 DownTo 0

                         WREG = LCD_Read8()  :  Lcd_S1.Byte0 = WREG >> 3
                         WREG = LCD_Read8()  :  WREG = WREG & $FC  :  Lcd_S2 = WREG * 8
                         WREG = LCD_Read8()  :  WREG = WREG & $F8  :  Lcd_S1.Byte1 = WREG
           
                         POSTDEC2 = Lcd_S1.Byte1 | Lcd_S2.Byte1
                         POSTDEC2 = Lcd_S1.Byte0 | Lcd_S2.Byte0
                        Next
                   Else

                        For Lcd_S5 = Lcd_S5 - 1 DownTo 0

                         WREG = LCD_Read8()  :  Lcd_S1.Byte0 = WREG >> 3
                         WREG = LCD_Read8()  :  WREG = WREG & $FC  :  Lcd_S2 = WREG * 8
                         WREG = LCD_Read8()  :  WREG = WREG & $F8  :  Lcd_S1.Byte1 = WREG
         
                         POSTINC2 = Lcd_S1.Byte0 | Lcd_S2.Byte0
                         POSTINC2 = Lcd_S1.Byte1 | Lcd_S2.Byte1
                        Next 
     
    End If
     
    LCD_End()
    
    LCD_DirWrite     

    LCD_Init()
    LCD_Rotation(LCD_Rot)              
$endif

EndProc



'****************************************************************************
$endif
